==> racer-0.1.1.tgz: package/lib/mixin.refs/index.js <==
var RefHelper, alreadySeen, arrayMutators, hasKeys, indiciesToIds, mutators, specIdentifier;
var __slice = Array.prototype.slice;
specIdentifier = require('../specHelper').identifier;
hasKeys = require('../util').hasKeys;
mutators = {};
arrayMutators = {};
module.exports = {
  init: function() {
    return this._refHelper = new RefHelper(this);
  },
  proto: {
    ref: function(ref, key) {
      if (key != null) {
        return {
          $r: ref,
          $k: key
        };
      } else {
        return {
          $r: ref
        };
      }
    },
    arrayRef: function(ref, key) {
      return {
        $r: ref,
        $k: key,
        $t: 'array'
      };
    },
    _dereference: function(path, data) {
      this._adapter.get(path, data || (data = this._specModel()));
      if (data.$remainder) {
        return data.$path + '.' + data.$remainder;
      } else {
        return data.$path;
      }
    }
  },
  onMixin: function(_mutators) {
    var fn, mutator, _results;
    mutators = _mutators;
    _results = [];
    for (mutator in _mutators) {
      fn = _mutators[mutator];
      _results.push(fn.type === 'array' ? arrayMutators[mutator] = fn : void 0);
    }
    return _results;
  }
};
RefHelper = function(model) {
  var adapter, eachNode, method, refHelper, _fn;
  this._model = model;
  this._adapter = adapter = model._adapter;
  refHelper = this;
  model.on('beforeTxn', function(method, args) {
    var $k, $r, arg, data, i, id, ids, index, indexArgs, keyId, keyObj, path, refObj, _i, _len, _len2, _ref, _results;
    if ((path = args[0]) == null) {
      return;
    }
    data = model._specModel();
    if (arrayMutators[method] && (refObj = adapter.getRef(path, data)) && refObj.$t === 'array') {
      $r = refObj.$r, $k = refObj.$k;
      $k = model._dereference($k, data);
      if (indexArgs = arrayMutators[method].indexArgs) {
        ids = {};
        keyObj = adapter.get($k, data);
        for (_i = 0, _len = indexArgs.length; _i < _len; _i++) {
          i = indexArgs[_i];
          if ((id = (_ref = args[i]) != null ? _ref.id : void 0) == null) {
            continue;
          }
          ids[i] = id;
          args.meta = {
            ids: ids
          };
          for (index = 0, _len2 = keyObj.length; index < _len2; index++) {
            keyId = keyObj[index];
            if (keyId == id) {
              args[i] = index;
              break;
            }
          }
        }
      }
      args[0] = path = $k;
      if (i = mutators[method].insertArgs) {
        _results = [];
        while (arg = args[i]) {
          if (id = arg.id) {
            if (hasKeys(arg, 'id')) {
              model.set($r + '.' + id, arg);
            }
            args[i] = id;
          } else {
            throw Error('arrayRef mutators require an id');
          }
          _results.push(i++);
        }
        return _results;
      }
    } else {
      return args[0] = model._dereference(path, data);
    }
  });
  _fn = function(method) {
    return model.on(method, function(args, isLocal, meta) {
      return refHelper.notifyPointersTo(method, args, isLocal, meta);
    });
  };
  for (method in mutators) {
    _fn(method);
  }
  eachNode = function(path, value, callback) {
    var nodePath, prop, val, _results;
    callback(path, value);
    _results = [];
    for (prop in value) {
      val = value[prop];
      nodePath = "" + path + "." + prop;
      _results.push(Object === (val != null ? val.constructor : void 0) ? eachNode(nodePath, val, callback) : callback(nodePath, val));
    }
    return _results;
  };
  model.on('setPre', function(_arg, ver, data) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return eachNode(path, value, function(path, value) {
      if (value && value.$r) {
        return refHelper.$indexRefs(path, value.$r, value.$k, value.$t, ver, data);
      }
    });
  });
  model.on('setPost', function(_arg, ver, data) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return eachNode(path, value, function(path, value) {
      return refHelper.updateRefsForKey(path, ver, data);
    });
  });
  model.on('delPost', function(_arg, ver, data) {
    var path;
    path = _arg[0];
    if (refHelper.isPathPointedTo(path, data)) {
      return refHelper.cleanupPointersTo(path, ver, data);
    }
  });
  for (method in arrayMutators) {
    model.on(method + 'Post', function(args, ver, data, meta) {
      var path, refObj;
      path = args[0];
      data || (data = model._specModel());
      if ((refObj = adapter.getRef(path, data)) && refObj.$t === 'array') {
        indiciesToIds(args, meta);
      }
      return refHelper.updateRefsForKey(path, ver, data);
    });
  }
};
RefHelper.prototype = {
  isPathPointedTo: function(path, data) {
    var found;
    found = this._adapter.get("$refs." + path + ".$", data);
    return found !== void 0;
  },
  $indexRefs: function(path, ref, key, type, ver, data) {
    var adapter, entry, keyVal, oldKey, oldRefObj, refs, refsKey, refsKeys, self;
    adapter = this._adapter;
    self = this;
    oldRefObj = adapter.getRef(path, data);
    if (key) {
      entry = [ref, key];
      if (type) {
        entry.push(type);
      }
      adapter.getAddPath("$keys." + key + ".$", data, ver, 'object')[path] = entry;
      keyVal = adapter.get(key, data);
      if (type === void 0 && keyVal === void 0) {
        return;
      }
      if (type === 'array') {
        keyVal = adapter.getAddPath(key, data, ver, 'array');
        refsKeys = keyVal.map(function(keyValMem) {
          return ref + '.' + keyValMem;
        });
        this._removeOld$refs(oldRefObj, path, ver, data);
        return refsKeys.forEach(function(refsKey) {
          return self._update$refs(refsKey, path, ref, key, type, ver, data);
        });
      }
      refsKey = ref + '.' + keyVal;
    } else {
      if (oldRefObj && (oldKey = oldRefObj.$k)) {
        refs = adapter.get("$keys." + oldKey + ".$", data);
        if (refs && refs[path]) {
          delete refs[path];
          if (!hasKeys(refs, specIdentifier)) {
            adapter.del("$keys." + oldKey, ver, data);
          }
        }
      }
      refsKey = ref;
    }
    this._removeOld$refs(oldRefObj, path, ver, data);
    return this._update$refs(refsKey, path, ref, key, type, ver, data);
  },
  _removeOld$refs: function(oldRefObj, path, ver, data) {
    var oldKey, oldKeyVal, oldRef, refHelper;
    if (oldRefObj && (oldRef = oldRefObj.$r)) {
      if (oldKey = oldRefObj.$k) {
        oldKeyVal = this._adapter.get(oldKey, data);
      }
      if (oldKey && (oldRefObj.$t === 'array')) {
        refHelper = this;
        oldKeyVal.forEach(function(oldKeyMem) {
          return refHelper._removeFrom$refs(oldRef, oldKeyMem, path, ver, data);
        });
        return this._removeFrom$refs(oldRef, void 0, path, ver, data);
      } else {
        return this._removeFrom$refs(oldRef, oldKeyVal, path, ver, data);
      }
    }
  },
  _removeFrom$refs: function(ref, key, path, ver, data) {
    var refEntries, refWithKey;
    if (key) {
      refWithKey = ref + '.' + key;
    }
    refEntries = this._adapter.get("$refs." + refWithKey + ".$", data);
    if (!refEntries) {
      return;
    }
    delete refEntries[path];
    if (!hasKeys(refEntries, specIdentifier)) {
      return this._adapter.del("$refs." + ref, ver, data);
    }
  },
  _update$refs: function(refsKey, path, ref, key, type, ver, data) {
    var entry;
    entry = [ref, key];
    if (type) {
      entry.push(type);
    }
    return this._adapter.getAddPath("$refs." + refsKey + ".$", data, ver, 'object')[path] = entry;
  },
  updateRefsForKey: function(path, ver, data) {
    var refs, self;
    self = this;
    if (refs = this._adapter.get("$keys." + path + ".$", data)) {
      this._eachValidRef(refs, data, function(path, ref, key, type) {
        return self.$indexRefs(path, ref, key, type, ver, data);
      });
    }
    return this.eachValidRefPointingTo(path, data, function(pointingPath, targetPathRemainder, ref, key, type) {
      return self.updateRefsForKey(pointingPath + '.' + targetPathRemainder, ver, data);
    });
  },
  _eachValidRef: function(refs, data, callback) {
    var key, o, path, ref, type, _ref, _results;
    _results = [];
    for (path in refs) {
      _ref = refs[path], ref = _ref[0], key = _ref[1], type = _ref[2];
      if (path === specIdentifier) {
        continue;
      }
      o = this._adapter.getRef(path, data);
      _results.push(o && o.$r === ref && o.$k == key ? callback(path, ref, key, type) : delete refs[path]);
    }
    return _results;
  },
  _eachRefSetPointingTo: function(path, refs, fn) {
    var i, prop, props, refPos, refSet, _results;
    i = 0;
    refPos = refs;
    props = path.split('.');
    _results = [];
    while (prop = props[i++]) {
      if (!(refPos = refPos[prop])) {
        return;
      }
      _results.push((refSet = refPos.$) ? fn(refSet, props.slice(i).join('.'), prop) : void 0);
    }
    return _results;
  },
  eachValidRefPointingTo: function(targetPath, data, fn) {
    var refs, self;
    if (!(refs = this._adapter.get('$refs', data))) {
      return;
    }
    self = this;
    return self._eachRefSetPointingTo(targetPath, refs, function(refSet, targetPathRemainder, possibleIndex) {
      return self._eachValidRef(refSet, data, function(pointingPath, ref, key, type) {
        if (type === 'array') {
          targetPathRemainder = possibleIndex + '.' + targetPathRemainder;
        }
        return fn(pointingPath, targetPathRemainder, ref, key, type);
      });
    });
  },
  eachArrayRefKeyedBy: function(path, data, fn) {
    var key, ref, refSet, refs, type, _ref, _results;
    if (!(refs = this._adapter.get('$keys', data))) {
      return;
    }
    refSet = (path + '.$').split('.').reduce(function(refSet, prop) {
      return refSet && refSet[prop];
    }, refs);
    if (!refSet) {
      return;
    }
    _results = [];
    for (path in refSet) {
      _ref = refSet[path], ref = _ref[0], key = _ref[1], type = _ref[2];
      _results.push(type === 'array' ? fn(path, ref, key) : void 0);
    }
    return _results;
  },
  notifyPointersTo: function(method, _arg, isLocal, meta) {
    var adapter, args, data, ignoreRoots, model, targetPath;
    targetPath = _arg[0], args = 2 <= _arg.length ? __slice.call(_arg, 1) : [];
    model = this._model;
    adapter = this._adapter;
    data = model._specModel();
    ignoreRoots = [];
    this.eachValidRefPointingTo(targetPath, data, function(pointingPath, targetPathRemainder, ref, key, type) {
      var id, index, keyArr, rest, _ref;
      if (type !== 'array') {
        if (alreadySeen(pointingPath, ref, ignoreRoots)) {
          return;
        }
        if (targetPathRemainder) {
          pointingPath += '.' + targetPathRemainder;
        }
      } else if (targetPathRemainder) {
        _ref = targetPathRemainder.split('.'), id = _ref[0], rest = 2 <= _ref.length ? __slice.call(_ref, 1) : [];
        keyArr = adapter.get(key, data);
        index = keyArr.indexOf(id);
        index = index === -1 ? keyArr.indexOf(parseInt(id, 10)) : index;
        if (index !== -1) {
          pointingPath += '.' + index;
          if (rest.length) {
            pointingPath += '.' + rest.join('.');
          }
        }
      }
      return model.emit(method, [pointingPath].concat(__slice.call(args)), isLocal, meta);
    });
    return this.eachArrayRefKeyedBy(targetPath, data, function(pointingPath, ref, key) {
      var i, obj;
      args = [pointingPath].concat(__slice.call(args));
      if (i = mutators[method].insertArgs) {
        obj = adapter.get(ref, data);
        while ((key = args[i]) != null) {
          args[i++] = obj[key];
        }
      }
      indiciesToIds(args, meta);
      return model.emit(method, args, isLocal, meta);
    });
  },
  cleanupPointersTo: function(path, ver, data) {
    var adapter, key, keyMem, keyVal, pointingPath, ref, refs, _ref, _results;
    adapter = this._adapter;
    refs = adapter.get("$refs." + path + ".$", data);
    if (refs === void 0) {
      return;
    }
    _results = [];
    for (pointingPath in refs) {
      _ref = refs[pointingPath], ref = _ref[0], key = _ref[1];
      keyVal = key && adapter.get(key, data);
      _results.push(keyVal && Array.isArray(keyVal) ? (keyMem = path.substr(ref.length + 1, pointingPath.length), adapter.remove(key, keyVal.indexOf(keyMem), 1, null, data), this.updateRefsForKey(key, ver, data)) : void 0);
    }
    return _results;
  }
};
alreadySeen = function(pointingPath, ref, ignoreRoots) {
  var root;
  for (root in ignoreRoots) {
    if (root === pointingPath.substr(0, root.length)) {
      return true;
    }
  }
  ignoreRoots.push(ref);
  return false;
};
indiciesToIds = function(args, meta) {
  var i, id, ids, _results;
  if (ids = meta != null ? meta.ids : void 0) {
    _results = [];
    for (i in ids) {
      id = ids[i];
      _results.push(args[i] = {
        id: id,
        index: args[i]
      });
    }
    return _results;
  }
};

==> racer-0.1.2.tgz: package/lib/mixin.refs/index.js <==
var RefHelper, alreadySeen, arrayMutators, hasKeys, indiciesToIds, mutators, specIdentifier;
var __slice = Array.prototype.slice;
specIdentifier = require('../specHelper').identifier;
hasKeys = require('../util').hasKeys;
mutators = {};
arrayMutators = {};
module.exports = {
  init: function() {
    return this._refHelper = new RefHelper(this);
  },
  proto: {
    ref: function(ref, key) {
      if (key != null) {
        return {
          $r: ref,
          $k: key
        };
      } else {
        return {
          $r: ref
        };
      }
    },
    arrayRef: function(ref, key) {
      return {
        $r: ref,
        $k: key,
        $t: 'array'
      };
    },
    _dereference: function(path, data) {
      this._adapter.get(path, data || (data = this._specModel()));
      if (data.$remainder) {
        return data.$path + '.' + data.$remainder;
      } else {
        return data.$path;
      }
    }
  },
  onMixin: function(_mutators) {
    var fn, mutator, _results;
    mutators = _mutators;
    _results = [];
    for (mutator in _mutators) {
      fn = _mutators[mutator];
      _results.push(fn.type === 'array' ? arrayMutators[mutator] = fn : void 0);
    }
    return _results;
  }
};
RefHelper = function(model) {
  var adapter, eachNode, method, refHelper, _fn;
  this._model = model;
  this._adapter = adapter = model._adapter;
  refHelper = this;
  model.on('beforeTxn', function(method, args) {
    var $k, $r, arg, data, i, id, ids, index, indexArgs, keyId, keyObj, path, refObj, _i, _len, _len2, _ref, _results;
    if ((path = args[0]) == null) {
      return;
    }
    data = model._specModel();
    if (arrayMutators[method] && (refObj = adapter.getRef(path, data)) && refObj.$t === 'array') {
      $r = refObj.$r, $k = refObj.$k;
      $k = model._dereference($k, data);
      if (indexArgs = arrayMutators[method].indexArgs) {
        ids = {};
        keyObj = adapter.get($k, data);
        for (_i = 0, _len = indexArgs.length; _i < _len; _i++) {
          i = indexArgs[_i];
          if ((id = (_ref = args[i]) != null ? _ref.id : void 0) == null) {
            continue;
          }
          ids[i] = id;
          args.meta = {
            ids: ids
          };
          for (index = 0, _len2 = keyObj.length; index < _len2; index++) {
            keyId = keyObj[index];
            if (keyId == id) {
              args[i] = index;
              break;
            }
          }
        }
      }
      args[0] = path = $k;
      if (i = mutators[method].insertArgs) {
        _results = [];
        while (arg = args[i]) {
          if (id = arg.id) {
            if (hasKeys(arg, 'id')) {
              model.set($r + '.' + id, arg);
            }
            args[i] = id;
          } else {
            throw Error('arrayRef mutators require an id');
          }
          _results.push(i++);
        }
        return _results;
      }
    } else {
      return args[0] = model._dereference(path, data);
    }
  });
  _fn = function(method) {
    return model.on(method, function(args, isLocal, _with, meta) {
      return refHelper.notifyPointersTo(method, args, isLocal, _with, meta);
    });
  };
  for (method in mutators) {
    _fn(method);
  }
  eachNode = function(path, value, callback) {
    var nodePath, prop, val, _results;
    callback(path, value);
    _results = [];
    for (prop in value) {
      val = value[prop];
      nodePath = "" + path + "." + prop;
      _results.push(Object === (val != null ? val.constructor : void 0) ? eachNode(nodePath, val, callback) : callback(nodePath, val));
    }
    return _results;
  };
  model.on('setPre', function(_arg, ver, data) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return eachNode(path, value, function(path, value) {
      if (value && value.$r) {
        return refHelper.$indexRefs(path, value.$r, value.$k, value.$t, ver, data);
      }
    });
  });
  model.on('setPost', function(_arg, ver, data) {
    var path, value;
    path = _arg[0], value = _arg[1];
    return eachNode(path, value, function(path, value) {
      return refHelper.updateRefsForKey(path, ver, data);
    });
  });
  model.on('delPost', function(_arg, ver, data) {
    var path;
    path = _arg[0];
    if (refHelper.isPathPointedTo(path, data)) {
      return refHelper.cleanupPointersTo(path, ver, data);
    }
  });
  for (method in arrayMutators) {
    model.on(method + 'Post', function(args, ver, data, meta) {
      var path, refObj;
      path = args[0];
      data || (data = model._specModel());
      if ((refObj = adapter.getRef(path, data)) && refObj.$t === 'array') {
        indiciesToIds(args, meta);
      }
      return refHelper.updateRefsForKey(path, ver, data);
    });
  }
};
RefHelper.prototype = {
  isPathPointedTo: function(path, data) {
    var found;
    found = this._adapter.get("$refs." + path + ".$", data);
    return found !== void 0;
  },
  $indexRefs: function(path, ref, key, type, ver, data) {
    var adapter, entry, i, keyVal, oldKey, oldRefObj, refs, refsKey, _i, _len;
    adapter = this._adapter;
    oldRefObj = adapter.getRef(path, data);
    if (key) {
      entry = [ref, key];
      if (type) {
        entry.push(type);
      }
      adapter.getAddPath("$keys." + key + ".$", data, ver, 'object')[path] = entry;
      keyVal = adapter.get(key, data);
      if (type === void 0 && keyVal === void 0) {
        return;
      }
      if (type === 'array') {
        keyVal = adapter.getAddPath(key, data, ver, 'array');
        this._removeOld$refs(oldRefObj, path, ver, data);
        for (_i = 0, _len = keyVal.length; _i < _len; _i++) {
          i = keyVal[_i];
          refsKey = ref + '.' + i;
          this._update$refs(refsKey, path, ref, key, type, ver, data);
        }
        return;
      }
      refsKey = ref + '.' + keyVal;
    } else {
      if (oldRefObj && (oldKey = oldRefObj.$k)) {
        refs = adapter.get("$keys." + oldKey + ".$", data);
        if (refs && refs[path]) {
          delete refs[path];
          if (!hasKeys(refs, specIdentifier)) {
            adapter.del("$keys." + oldKey, ver, data);
          }
        }
      }
      refsKey = ref;
    }
    this._removeOld$refs(oldRefObj, path, ver, data);
    return this._update$refs(refsKey, path, ref, key, type, ver, data);
  },
  _removeOld$refs: function(oldRefObj, path, ver, data) {
    var oldKey, oldKeyMem, oldKeyVal, oldRef, _i, _len;
    if (oldRefObj && (oldRef = oldRefObj.$r)) {
      if (oldKey = oldRefObj.$k) {
        oldKeyVal = this._adapter.get(oldKey, data);
      }
      if (oldKey && (oldRefObj.$t === 'array')) {
        for (_i = 0, _len = oldKeyVal.length; _i < _len; _i++) {
          oldKeyMem = oldKeyVal[_i];
          this._removeFrom$refs(oldRef, oldKeyMem, path, ver, data);
        }
        return this._removeFrom$refs(oldRef, void 0, path, ver, data);
      } else {
        return this._removeFrom$refs(oldRef, oldKeyVal, path, ver, data);
      }
    }
  },
  _removeFrom$refs: function(ref, key, path, ver, data) {
    var refEntries, refWithKey;
    if (key) {
      refWithKey = ref + '.' + key;
    }
    refEntries = this._adapter.get("$refs." + refWithKey + ".$", data);
    if (!refEntries) {
      return;
    }
    delete refEntries[path];
    if (!hasKeys(refEntries, specIdentifier)) {
      return this._adapter.del("$refs." + ref, ver, data);
    }
  },
  _update$refs: function(refsKey, path, ref, key, type, ver, data) {
    var entry;
    entry = [ref, key];
    if (type) {
      entry.push(type);
    }
    return this._adapter.getAddPath("$refs." + refsKey + ".$", data, ver, 'object')[path] = entry;
  },
  updateRefsForKey: function(path, ver, data) {
    var refs, self;
    self = this;
    if (refs = this._adapter.get("$keys." + path + ".$", data)) {
      this._eachValidRef(refs, data, function(path, ref, key, type) {
        return self.$indexRefs(path, ref, key, type, ver, data);
      });
    }
    return this.eachValidRefPointingTo(path, data, function(pointingPath, targetPathRemainder, ref, key, type) {
      return self.updateRefsForKey(pointingPath + '.' + targetPathRemainder, ver, data);
    });
  },
  _eachValidRef: function(refs, data, callback) {
    var key, o, path, ref, type, _ref, _results;
    _results = [];
    for (path in refs) {
      _ref = refs[path], ref = _ref[0], key = _ref[1], type = _ref[2];
      if (path === specIdentifier) {
        continue;
      }
      o = this._adapter.getRef(path, data);
      _results.push(o && o.$r === ref && o.$k == key ? callback(path, ref, key, type) : delete refs[path]);
    }
    return _results;
  },
  _eachRefSetPointingTo: function(path, refs, fn) {
    var i, prop, props, refPos, refSet, _results;
    i = 0;
    refPos = refs;
    props = path.split('.');
    _results = [];
    while (prop = props[i++]) {
      if (!(refPos = refPos[prop])) {
        return;
      }
      _results.push((refSet = refPos.$) ? fn(refSet, props.slice(i).join('.'), prop) : void 0);
    }
    return _results;
  },
  eachValidRefPointingTo: function(targetPath, data, fn) {
    var refs, self;
    if (!(refs = this._adapter.get('$refs', data))) {
      return;
    }
    self = this;
    return self._eachRefSetPointingTo(targetPath, refs, function(refSet, targetPathRemainder, possibleIndex) {
      return self._eachValidRef(refSet, data, function(pointingPath, ref, key, type) {
        if (type === 'array') {
          targetPathRemainder = possibleIndex + '.' + targetPathRemainder;
        }
        return fn(pointingPath, targetPathRemainder, ref, key, type);
      });
    });
  },
  eachArrayRefKeyedBy: function(path, data, fn) {
    var key, ref, refSet, refs, type, _ref, _results;
    if (!(refs = this._adapter.get('$keys', data))) {
      return;
    }
    refSet = (path + '.$').split('.').reduce(function(refSet, prop) {
      return refSet && refSet[prop];
    }, refs);
    if (!refSet) {
      return;
    }
    _results = [];
    for (path in refSet) {
      _ref = refSet[path], ref = _ref[0], key = _ref[1], type = _ref[2];
      _results.push(type === 'array' ? fn(path, ref, key) : void 0);
    }
    return _results;
  },
  notifyPointersTo: function(method, _arg, isLocal, _with, meta) {
    var adapter, args, data, ignoreRoots, model, targetPath;
    targetPath = _arg[0], args = 2 <= _arg.length ? __slice.call(_arg, 1) : [];
    model = this._model;
    adapter = this._adapter;
    data = model._specModel();
    ignoreRoots = [];
    this.eachValidRefPointingTo(targetPath, data, function(pointingPath, targetPathRemainder, ref, key, type) {
      var id, index, keyArr, rest, _ref;
      if (type !== 'array') {
        if (alreadySeen(pointingPath, ref, ignoreRoots)) {
          return;
        }
        if (targetPathRemainder) {
          pointingPath += '.' + targetPathRemainder;
        }
      } else if (targetPathRemainder) {
        _ref = targetPathRemainder.split('.'), id = _ref[0], rest = 2 <= _ref.length ? __slice.call(_ref, 1) : [];
        keyArr = adapter.get(key, data);
        index = keyArr.indexOf(id);
        index = index === -1 ? keyArr.indexOf(parseInt(id, 10)) : index;
        if (index !== -1) {
          pointingPath += '.' + index;
          if (rest.length) {
            pointingPath += '.' + rest.join('.');
          }
        }
      }
      return model.emit(method, [pointingPath].concat(__slice.call(args)), isLocal, _with, meta);
    });
    return this.eachArrayRefKeyedBy(targetPath, data, function(pointingPath, ref, key) {
      var i, obj;
      args = [pointingPath].concat(__slice.call(args));
      if (i = mutators[method].insertArgs) {
        obj = adapter.get(ref, data);
        while ((key = args[i]) != null) {
          args[i++] = obj[key];
        }
      }
      indiciesToIds(args, meta);
      return model.emit(method, args, isLocal, _with, meta);
    });
  },
  cleanupPointersTo: function(path, ver, data) {
    var adapter, key, keyMem, keyVal, pointingPath, ref, refs, _ref, _results;
    adapter = this._adapter;
    refs = adapter.get("$refs." + path + ".$", data);
    if (refs === void 0) {
      return;
    }
    _results = [];
    for (pointingPath in refs) {
      _ref = refs[pointingPath], ref = _ref[0], key = _ref[1];
      keyVal = key && adapter.get(key, data);
      _results.push(keyVal && Array.isArray(keyVal) ? (keyMem = path.substr(ref.length + 1, pointingPath.length), adapter.remove(key, keyVal.indexOf(keyMem), 1, null, data), this.updateRefsForKey(key, ver, data)) : void 0);
    }
    return _results;
  }
};
alreadySeen = function(pointingPath, ref, ignoreRoots) {
  var root;
  for (root in ignoreRoots) {
    if (root === pointingPath.substr(0, root.length)) {
      return true;
    }
  }
  ignoreRoots.push(ref);
  return false;
};
indiciesToIds = function(args, meta) {
  var i, id, ids, _results;
  if (ids = meta != null ? meta.ids : void 0) {
    _results = [];
    for (i in ids) {
      id = ids[i];
      _results.push(args[i] = {
        id: id,
        index: args[i]
      });
    }
    return _results;
  }
};
